"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.subscribe = void 0;
const subscribe = (...subscribables) => {
    return (component) => {
        class SubscribedComponent extends component {
            constructor(...args) {
                super(...args);
                const forceUpdateWrapper = () => this.forceUpdate();
                const initialComponentDidMount = this.componentDidMount;
                const initialComponentWillUnmount = this.componentWillUnmount;
                const initialRender = this.render;
                let updatedWhileRender = false;
                const updatedWhileRenderSetter = () => updatedWhileRender = true;
                this.componentDidMount = () => {
                    for (const wrapper of subscribables)
                        wrapper.unsubscribe(updatedWhileRenderSetter);
                    if (updatedWhileRender)
                        this.forceUpdate();
                    for (const wrapper of subscribables)
                        wrapper.subscribe(forceUpdateWrapper);
                    if (initialComponentDidMount)
                        initialComponentDidMount();
                };
                this.componentWillUnmount = () => {
                    for (const wrapper of subscribables)
                        wrapper.unsubscribe(forceUpdateWrapper);
                    if (initialComponentWillUnmount)
                        initialComponentWillUnmount();
                };
                this.render = () => {
                    for (const wrapper of subscribables)
                        wrapper.subscribe(updatedWhileRenderSetter);
                    return initialRender();
                };
            }
        }
        Object.defineProperty(SubscribedComponent, "name", { value: component.name });
        Object.defineProperty(SubscribedComponent, "displayName", { value: `(Subscribed)(${component.name})` });
        return SubscribedComponent;
    };
};
exports.subscribe = subscribe;
//# sourceMappingURL=subscribe.js.map