"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Field = void 0;
class Field {
    constructor(form, name, initialValue) {
        this.setValidators = (validators) => {
            if (this.validators[0] !== validators[0]) {
                this.validators = validators;
                this.validate();
            }
        };
        this.setSubmissionErrors = (errors) => {
            this.errors = errors;
            this.submitFailed = true;
            this.trigger();
        };
        this.setInitialValue = (value) => {
            if (this.initialValue !== value && value !== undefined) {
                this.initialValue = value;
                if (this.value === undefined) {
                    this.value = value;
                    this.inputProps.value = value !== null && value !== void 0 ? value : "";
                    this.form.setValue(this.name, value);
                    this.validate();
                    this.trigger();
                }
            }
        };
        this.setDefaultValue = (value) => {
            if (this.defaultValue !== value && value !== undefined) {
                this.defaultValue = value;
                if (this.initialValue === undefined)
                    this.setInitialValue(value);
            }
        };
        this.validate = () => {
            const errors = [];
            for (const validator of this.validators) {
                const result = validator(this.value);
                if (result !== undefined)
                    errors.push(result);
            }
            this.errors = errors;
            return errors.length === 0;
        };
        this.focus = () => {
            this.active = true;
            this.visited = true;
            this.trigger();
        };
        this.change = (value) => {
            this.value = value;
            this.submitFailed = false;
            this.inputProps.value = value !== null && value !== void 0 ? value : "";
            this.form.setValue(this.name, value);
            this.validate();
            this.trigger();
        };
        this.blur = () => {
            this.active = false;
            this.touched = this.visited;
            this.trigger();
        };
        this.reset = () => {
            var _a;
            if (this.value !== this.defaultValue) {
                this.value = this.defaultValue;
                this.inputProps.value = (_a = this.defaultValue) !== null && _a !== void 0 ? _a : "";
                if (this.form.valueExists(this.name))
                    this.form.setValue(this.name, this.defaultValue);
            }
            this.visited = false;
            this.touched = false;
            this.submitFailed = false;
            this.validate();
            this.trigger();
        };
        this.push = (value = null) => {
            var _a;
            this.change([...((_a = this.value) !== null && _a !== void 0 ? _a : []), value]);
        };
        this.getKeysListsFlags = () => {
            const keys = this.form.getKeys();
            const keysList = [];
            const keysFlags = {};
            for (const key of keys)
                if (key.startsWith(this.name + "[")) {
                    keysList.push(key);
                    keysFlags[key] = false;
                }
            return [keysList, keysFlags];
        };
        this.import = (field) => {
            var _a;
            this.validators = field.validators;
            this.errors = field.errors;
            this.initialValue = field.initialValue;
            this.defaultValue = field.defaultValue;
            this.value = field.value;
            this.active = field.active;
            this.visited = field.visited;
            this.touched = field.touched;
            this.submitFailed = field.submitFailed;
            this.inputProps = Object.assign(Object.assign({}, this.inputProps), { value: (_a = field.value) !== null && _a !== void 0 ? _a : "" });
            this.trigger();
        };
        this.remove = (index) => {
            const [keysList, keysFlags] = this.getKeysListsFlags();
            for (let n = index; n < this.value.length; n += 1) {
                if (n < this.value.length - 1)
                    for (const key of keysList) {
                        const prefix = `${this.name}[${n + 1}]`;
                        if (key.startsWith(prefix)) {
                            const name = key.substring(prefix.length);
                            const keyTarget = `${this.name}[${n}]${name}`;
                            this.form.getField(keyTarget).import(this.form.getField(key));
                            keysFlags[keyTarget] = true;
                        }
                    }
                for (const key of keysList) {
                    const prefix = `${this.name}[${n}]`;
                    if (key.startsWith(prefix))
                        if (keysFlags[key] === false)
                            this.form.removeField(key);
                }
            }
            this.value.splice(index, 1);
            this.change(this.value.length > 0 ? this.value : undefined);
        };
        this.subscribe = (subscriber) => {
            this.subscribers.add(subscriber);
        };
        this.unsubscribe = (subscriber) => {
            this.subscribers.delete(subscriber);
        };
        this.trigger = () => {
            for (const subscriber of this.subscribers)
                subscriber(this.value);
        };
        this.form = form;
        this.name = name;
        this.validators = [];
        this.errors = [];
        this.initialValue = initialValue;
        this.defaultValue = undefined;
        this.value = initialValue;
        this.active = false;
        this.visited = false;
        this.touched = false;
        this.submitFailed = false;
        this.inputProps = {
            name: name,
            onChange: (event) => this.change(event.target.value),
            onFocus: () => this.focus(),
            onBlur: () => this.blur(),
            value: initialValue !== null && initialValue !== void 0 ? initialValue : "",
        };
        this.subscribers = new Set();
    }
}
exports.Field = Field;
//# sourceMappingURL=field.js.map