"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.remove = exports.set = exports.get = exports.exists = void 0;
var AccessKind;
(function (AccessKind) {
    AccessKind[AccessKind["Object"] = 0] = "Object";
    AccessKind[AccessKind["Array"] = 1] = "Array";
})(AccessKind || (AccessKind = {}));
;
const codes = {
    dot: '.'.charCodeAt(0),
    openingBracket: '['.charCodeAt(0),
    closingBracket: ']'.charCodeAt(0),
    zero: '0'.charCodeAt(0),
    nine: '9'.charCodeAt(0),
};
function stringToPathObject(string) {
    let path;
    let numbersOnly;
    let startIndex = string.length;
    let currentIndex = startIndex;
    while (currentIndex >= 0) {
        const charCode = string.charCodeAt(currentIndex);
        switch (charCode) {
            case codes.dot: {
                const accessKind = AccessKind.Object;
                const name = string.substring(currentIndex + 1, startIndex);
                path = { accessKind, name, next: path };
                startIndex = currentIndex;
                break;
            }
            case codes.openingBracket: {
                const accessKind = numbersOnly ? AccessKind.Array : AccessKind.Object;
                const index = string.substring(currentIndex + 1, startIndex - 1);
                path = { accessKind, index: numbersOnly ? Number(index) : index, next: path };
                startIndex = currentIndex;
                break;
            }
            case codes.closingBracket: {
                numbersOnly = true;
                break;
            }
            default: {
                if (charCode < codes.zero || charCode > codes.nine)
                    numbersOnly = false;
                break;
            }
        }
        currentIndex -= 1;
    }
    if (startIndex > 0) {
        const accessKind = AccessKind.Object;
        const name = string.substring(0, startIndex);
        path = { accessKind, name, next: path };
    }
    return path;
}
function existsInner(object, path) {
    const propertyName = path.accessKind === AccessKind.Object ? path.name : path.index;
    if (path.next !== undefined) {
        if (object[propertyName] === undefined)
            return false;
        return existsInner(object[propertyName], path.next);
    }
    else {
        return true;
    }
}
function exists(object, path) {
    let pathObject = stringToPathObject(path);
    return existsInner(object, pathObject);
}
exports.exists = exists;
function getInner(object, path) {
    const propertyName = path.accessKind === AccessKind.Object ? path.name : path.index;
    if (path.next !== undefined) {
        if (object[propertyName] === undefined)
            return undefined;
        return getInner(object[propertyName], path.next);
    }
    else {
        return object[propertyName];
    }
}
function get(object, path) {
    let pathObject = stringToPathObject(path);
    return getInner(object, pathObject);
}
exports.get = get;
function setInner(object, path, value) {
    var _a;
    const propertyName = path.accessKind === AccessKind.Object ? path.name : path.index;
    if (path.next !== undefined) {
        const propertyValue = (_a = object[propertyName]) !== null && _a !== void 0 ? _a : (path.next.accessKind === AccessKind.Array ? [] : {});
        setInner(object[propertyName] = propertyValue, path.next, value);
    }
    else {
        object[propertyName] = value;
    }
}
function set(object, path, value) {
    setInner(object, stringToPathObject(path), value);
}
exports.set = set;
function deleleProperty(path, object, propertyName) {
    if (path.accessKind !== AccessKind.Array)
        delete object[propertyName];
    else
        object[propertyName] = null;
}
function removeInner(object, path) {
    var _a;
    const propertyName = path.accessKind === AccessKind.Object ? path.name : path.index;
    if (path.next !== undefined) {
        const propertyValue = (_a = object[propertyName]) !== null && _a !== void 0 ? _a : (path.next.accessKind === AccessKind.Array ? [] : {});
        removeInner(object[propertyName] = propertyValue, path.next);
        if ((path.next.accessKind === AccessKind.Array ? object[propertyName] : Object.keys(object[propertyName])).length == 0)
            deleleProperty(path, object, propertyName);
    }
    else {
        deleleProperty(path, object, propertyName);
    }
}
function remove(object, path) {
    let pathObject = stringToPathObject(path);
    removeInner(object, pathObject);
}
exports.remove = remove;
//# sourceMappingURL=object.js.map